#!/bin/sh

ifcfg_loc="/etc/sysconfig/network-scripts"
iflist=`ls $ifcfg_loc/ifcfg-eth* 2>/dev/null`

# SUSE config location
if [ -z "$iflist" ]
then
    ifcfg_loc="/etc/sysconfig/network"
    iflist=`ls $ifcfg_loc/ifcfg-eth* 2>/dev/null`
fi

NET_STOPPED=0
check_netstop()
{
    if [ $NET_STOPPED -eq 0 ] ; then
	/etc/init.d/network stop
	NET_STOPPED=1
    fi
}

check_netstart()
{
    if [ $NET_STOPPED -ne 0 ] ; then
	/etc/init.d/network start
	NET_STOPPED=0
    fi
}

if [ "$1" == "uninstall" ]
then

    echo "Bridges uninstallation"

    if [ -f /etc/init.d/brfx ]
    then
	rm -f /etc/init.d/brfx
	ln -sf /etc/init.d/network /etc/rc0.d/???network
	ln -sf /etc/init.d/network /etc/rc1.d/???network
	ln -sf /etc/init.d/network /etc/rc2.d/???network
	ln -sf /etc/init.d/network /etc/rc3.d/???network
	ln -sf /etc/init.d/network /etc/rc4.d/???network
	ln -sf /etc/init.d/network /etc/rc6.d/???network
    fi

    iflist=`ls $ifcfg_loc/ifcfg-eth*.bak 2>/dev/null`
    for ifentry in $iflist
    do
	BRIDGE=""

	. $ifentry

	if [ ! -z $BRIDGE ]
	then
	    keep_brlist=$keep_brlist" $BRIDGE"
	fi

	check_netstop
	mv -f $ifentry `echo $ifentry | sed s@\.bak@@g`
    done

    brlist=`ls $ifcfg_loc/ifcfg-br* 2>/dev/null`
    for brentry in $brlist
    do
	brname=`basename $brentry`
	if [ -z `echo $keep_brlist | grep "$brname\ "` ]
	then
	    check_netstop
	    rm -f $brentry
	fi
    done

    check_netstart

    exit 0
fi


if [ -f /etc/fx_target ] ; then
    if [ `cat /etc/fx_target` == "bfx" ] ; then
	usebridge="yes"
    fi
fi

cd `/usr/bin/dirname $0`

install_brfx()
{
    cp -a -f sys/brfx /etc/init.d
    ln -sf /etc/init.d/brfx /etc/rc0.d/???network
    ln -sf /etc/init.d/brfx /etc/rc1.d/???network
    ln -sf /etc/init.d/brfx /etc/rc2.d/???network
    ln -sf /etc/init.d/brfx /etc/rc3.d/???network
    ln -sf /etc/init.d/brfx /etc/rc4.d/???network
    ln -sf /etc/init.d/brfx /etc/rc6.d/???network
}


setup_dev="look below"
setup_bridge()
{
    #Find device
    for ifentry in $iflist
    do
	if [ ! -z "`echo $ifentry | grep \.bak`" ] ; then
	    continue
	fi

	DEVICE=""
	IPADDR=""
	NETMASK=""
	LABEL=""
	BRIDGE=""
	FXBRIDGE=""
	BOOTPROTO=""
	BROADCAST=""
	GATEWAY=""
	IPV6INIT=""
	USERCTL=""
	PEERDNS=""

	. ${ifentry}

	# New style SUSE
	if [ -z "$DEVICE" ] ; then
	    if [ ! -z "$LABEL" ] ; then
		DEVICE=eth"$LABEL"
	    fi
	fi

	if [ $DEVICE == $setup_dev ]
	then
	    if [ ! -z "$BRIDGE" ]
	    then
		return
	    else
		# SUSE support
		if [ ! -z $FXBRIDGE ] ; then
		    return
		fi

		echo "Bridging interface $setup_dev"

		#Find first unused bridge (SUSE - need bridge script?)
		for br_ind in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20
		do
		    if [ ! -f "$ifcfg_loc/ifcfg-br"$br_ind ]
		    then

			if [ -f /etc/init.d/brfx ]; then
			    /etc/init.d/brfx stop
			else
			    if [ -f /etc/init.d/network ]; then
				/etc/init.d/network stop
			    fi
			fi
			echo Install br"$br_ind" on "$setup_dev"

			cp -a -f $ifentry "$ifentry".bak

			if [ ! -z "`cat $ifentry | grep LABEL`" ]
			then
			    # SUSE type distro
			    cat << EOF >"$ifcfg_loc/ifcfg-br"$br_ind
BOOTPROTO="$BOOTPROTO"
STARTMODE="onboot"
IPADDR="$IPADDR"
NETMASK="$NETMASK"
BROADCAST="$BROADCAST"
GATEWAY="$GATEWAY"

# Bridge parameter
STP=off
DELAY=3
EOF

			    echo -e "\nFXBRIDGE=br$br_ind" >> $ifentry

			    if [ ! -f /etc/init.d/brfx ]; then
				install_brfx
			    fi
			else
			    cat << EOF >"$ifcfg_loc/ifcfg-br"$br_ind
DEVICE=br$br_ind

BOOTPROTO=$BOOTPROTO
ONBOOT=yes
TYPE=Bridge
IPADDR=$IPADDR
NETMASK=$NETMASK
BROADCAST=$BROADCAST
GATEWAY=$GATEWAY

USERCTL=$USERCTL
PEERDNS=$PEERDNS
IPV6INIT=$IPV6INIT

# Bridge parameter
STP=off
DELAY=3
EOF

			    echo -e "\nBRIDGE=br$br_ind" >> $ifentry
			fi

			cat $ifentry | sed s@BROADCAST@#BROADCAST@g | sed s@GATEWAY@#GATEWAY@g | sed s@NETMASK@#NETMASK@g | sed s@IPADDR@#IPADDR@g | sed s@static@none@g > $ifentry.new
			mv -f ${ifentry}.new $ifentry

			if [ -f /etc/init.d/brfx ]; then
			    /etc/init.d/brfx start
			else
			    if [ -f /etc/init.d/network ]; then
				/etc/init.d/network start
			    else
				echo "Please reboot for changes to take effect."
			    fi
			fi

			return;
		    fi
		done

		echo "No free Bridge index."
	    fi
	fi
    done

    echo "Bridge wasn't installed! Please e-mail us at support@fx.dk"
    exit 1;
}

echo "Select the Network Adapter to install to (i.e. the external network):"

ifnum=0
OKAY=1
INTPRESENT=0

while [ $OKAY == 1 ]
do

    ifentry=`./intf-name $ifnum`
    
    let ifnum="$ifnum + 1"

    if [ $ifentry != "N/A" ]
    then
	ip=`ifconfig $ifentry 2> /dev/null | grep "inet addr" | awk '{ print $2 }' | sed s/addr://g`
	echo "  $ifnum: $ifentry - $ip"
	INTPRESENT=1
    else
	OKAY=0
    fi

done

if [ $INTPRESENT != 1 ]
then
    echo "No Network Adapters found!"
    sleep 3
    exit 1
fi

OKAY=0
while [ $OKAY -ne 1 ]
do
    read answ
    let ind="$answ - 1"

    ./intf-name $ind > /dev/null

    # Check if such iface exists
    if [ $? -eq 0 ]
    then
	OKAY=1
    else
	echo -e -n "Invalid selection.\nChoose the adapter to firewall:"
    fi
done

if [ -f "config/gateway.cnf" ]
then
    cp -a -f config/gateway.cnf config/gateway.bak
else
    if [ -f "config/gateway.cn_" ]
    then
	cp -a -f config/gateway.cn_ config/gateway.cnf
    else
	echo "No Injoy Firewall configuration found!"
	sleep 3
	exit 2
    fi
fi

if [ ! -z "$usebridge" ] ; then
    let setup_ind="$answ - 1"
    setup_dev="eth"$setup_ind
    setup_bridge
fi

tmpfile="config/gateway.new"
rm -f $tmpfile

cat config/gateway.cnf | while read line
do
    if [ ! -z "`echo $line | grep [\\;#]`" ] ; then
	echo $line>>$tmpfile
	continue
    fi
    if [ -z "`echo $line`" ] ; then
	echo >>$tmpfile
	continue
    fi
    if [ ! -z "`echo $line | grep Device-Index`" ] ; then
	if [ ! -z "`echo $line | grep SETTINGS`" ] ; then
	    echo "SETTINGS">>$tmpfile
	fi
    else
	if [ ! -z "`echo $line | grep SETTINGS`" ] ; then
	    echo -e "$line">>$tmpfile
	else
	    echo -e "\t\t$line">>$tmpfile
	fi
    fi
done

echo -e "\t\tDevice-Index=$answ">>$tmpfile

mv -f $tmpfile config/gateway.cnf
