#!/bin/sh

release=$1
file=$2
path=$3

if [ -z $release ]
then
    echo -e "Linux kernel version is not specified! Aborting"
    exit
fi

if [ -z $file ]
then
    echo -e "NIC module name is not specified! Aborting"
    exit
fi

if [ x"$path" == x ]
then
    path="kernel/drivers/net"
fi

if [ ! -z `echo ${release} | grep "2\.6"` ]
then
    mod_ext=ko
else
    mod_ext=o
fi

destfile=/lib/modules/"$release"/"$path"/"$file"
bakfile=/lib/modules/"$release"/"$path"/fx-bak/"$file"
tempfile=/lib/modules/"$release"/"$path"/tmp
fxfile=/lib/modules/"$release"/"$path"/fx."$mod_ext"

if [ ! -f "$destfile"."$mod_ext" ]
then
    echo -e "NIC module ${destfile}.${mod_ext} doesn't exist! Aborting"
    exit
fi

mkdir "/lib/modules/${release}/${path}/fx-bak" 2> /dev/null

for dir in `find /lib/modules/${release}/${path} -type d ! -path "*fx-bak*" -printf "%P\n"`
do
    mkdir "/lib/modules/${release}/${path}/fx-bak/${dir}" 2> /dev/null 
done

echo -n "Patching NIC modules: ${destfile}.${mod_ext} ... "

if [ -f "$bakfile".backup ]
then

    echo -n "restoring backup ... "
    cp -f "$bakfile".backup "$destfile"."$mod_ext"
    
fi    

cp -a "$destfile"."$mod_ext" "$destfile"_fx."$mod_ext"
cp -a "$destfile"."$mod_ext" "$bakfile".backup

sym_list="unregister_netdev unregister_netdevice register_netdev register_netdevice netif_rx init_etherdev ei_open ei_close ei"
if [ x"$mod_ext" = x"ko" ]
then
    sym_list=${sym_list}" netif_receive_skb"
fi

for sym in ${sym_list}
do
    fxsymname=`/bin/findsym $fxfile fx_${sym}`
    modsymname=`/bin/findsym ${destfile}_fx.${mod_ext} ${sym}`
    /bin/fx_objcopy --remove-section=.module_sig --redefine-sym ${modsymname}=${fxsymname} "$destfile"_fx."$mod_ext" $tempfile
    mv $tempfile "$destfile"_fx."$mod_ext"
done

rm -f "$destfile"."$mod_ext"
mv "$destfile"_fx."$mod_ext" "$destfile"."$mod_ext"

echo "OK"

