#!/bin/sh

#################################################
# Messages definition
#################################################

ABORT="> Aborting installation..."
EXIT="Exiting installation..."

CONTINUE="If you are sure you wish to continue with uninstallation, please type Y now."

RC_S="> Making changes to rc.local ..."

DEL_RCFX="> Removing rc.fx file ..."

FX_GATEWAY="> Removing fx.gateway file ..."

NET_SCRIPTS="> Making changes to 'network-scripts' system files ..."

RC_D="> Making changes to 'rc.d' system files ..."

DEL_STARTUP="> Removing the F/X start-up files"

DEL_PATCHED="> Removing patched drivers ..."

ALS_RESTORE="> Restoring interface aliases ..."

FOLDERS="> Removing shortcuts and folders ..."

COMPLETE=">>> For support, contact us at support@fx.dk\n
	 Uninstallation complete, please reboot for changes to take effect."	 

COMPLETE_BSD="Uninstall completed."	 

######## END OF DEFINITIONS #####################
release=`uname --release`
instdir=`pwd`
opsystem=`uname -s`
gnomedir="/usr/share/gnome/apps"
kdedir="/opt/kde/share/applnk"

if [ "$opsystem" = "FreeBSD" ]
then
	gnomedir="/usr/X11R6/share/gnome/apps"
fi

echo $CONTINUE
read answer
if [ "$answer" != "Y" ]
then
	echo $ABORT
	exit 1
fi                   	

if [ "$opsystem" = "Linux" ]
then

	if [ -f /etc/rc.d/rc.fx ]
	then
		echo $DEL_RCFX
		rm -f /etc/rc.d/rc.fx
	fi
	if [ -f /etc/rc.d/fx.gateway ]
	then
		echo $FX_GATEWAY
		rm -f /etc/rc.d/fx.gateway
	fi

	if [ -f /etc/rc.d/rc.local ]
	then
	     startup_check=`sed '$!d' /etc/rc.d/rc.local`
	     if [ "$startup_check" = "/etc/rc.d/rc.fx" ]
	     then
		   echo $RC_S
		   sed '$d' /etc/rc.d/rc.local >/etc/rc.d/rc.local.tmp
		   mv -f /etc/rc.d/rc.local.tmp /etc/rc.d/rc.local
		   chmod +x /etc/rc.d/rc.local
	     fi
	fi

	old_install=`find /etc -path "*ifcfg-fx*" ! -type d -printf "%P "`

	if [ ! -z "$old_install" ]
	then

	    echo $DEL_PATCHED
	
	    for file in `ls -d /lib/modules/2.*`
	    do
		if [ ! -z `echo $file | grep "2\.6"` ]
		then
		    mod_ext=ko
		else
		    mod_ext=o
		fi
		
		if [ -r "$file"/kernel/drivers/net/fx."$mod_ext" ]
    		then
		    echo "Removing ${file}/kernel/drivers/net/fx.{$mod_ext}:"
       		    rm -f "$file"/kernel/drivers/net/fx."$mod_ext"
    		fi

		if [ -r /etc/fx.conf ]
		then		
        	    for rmodfile in `cat /etc/fx.conf`
        	    do
            		if [ -r "$file"/kernel/drivers/net/"$rmodfile"_fx."$mod_ext" ]
            		then
                	    echo "Removing ${file}/kernel/drivers/net/${rmodfile}_fx.${mod_ext}:"
                	    rm -f "$file"/kernel/drivers/net/"$rmodfile"_fx."$mod_ext"
            		fi
        	    done
		fi
	    done
    
	fi

	echo $DEL_STARTUP
	
	rm -f /etc/init.d/gateway
	rm -f /etc/rc?.d/???gateway
	
	rm -f /etc/init.d/fx
	rm -f /etc/rc?.d/???fx

	rm -f /bin/fx_objcopy
	rm -f /bin/findsym
	rm -f /etc/sysconfig/network-scripts/ifcfg-fx*
	rm -f /etc/sysconfig/networking/devices/ifcfg-fx*
	rm -f /etc/sysconfig/networking/profiles/default/ifcfg-fx*
fi

"$instdir"/Unpatch-Drivers.sh $release

# SuSE specific drivers
#

if [ -f /etc/fx.conf ]
then
    if [ ! -z `cat /etc/fx.conf | grep "#extra"` ]
    then
	"$instdir"/Unpatch-Drivers.sh $release extra
    fi

    if [ ! -z `cat /etc/fx.conf | grep "#updates"` ]
    then
	"$instdir"/Unpatch-Drivers.sh $release updates
    fi
fi

if [ -f /etc/fx_start ]
then
	rm -f /etc/fx_start*
fi
if [ -f /etc/fx_forward ]
then
	rm -f /etc/fx_forward*
fi

if [ -f /etc/fx.conf ]
then
	rm -f /etc/fx.conf*
fi

./Adapter.sh uninstall

if [ -f /etc/fx_target ]
then
	rm -f /etc/fx_target*
fi
if [ -f /etc/fx.conf.bak ]
then
	rm -f /etc/fx.conf.bak*
fi


#### The block ends here

echo $FOLDERS
echo ""
echo ""

./menupars >/dev/null 2>/dev/null
if [ $? -eq 0 ] ; then
    menupars="./menupars"
else
    ./menupars2 >/dev/null 2>/dev/null
    if [ $? -eq 0 ] ; then
	menupars="./menupars2"
    fi
fi

if [ ! -z "$menupars" ]
then
    $menupars -c firewall.xml -d >/dev/null 2>/dev/null
    rc=$?
else
    rc=1
fi

if [ $rc -ne 0 ]
then
    rm -rf "${kdedir}/Internet/InJoy Firewall"
    rm -rf "${HOME}/Desktop/InJoy Firewall"
    rm -rf /tmp/.fx-kde

    rm -rf "${gnomedir}/Internet/InJoy Firewall/"
    rm -rf "${HOME}/.gnome-desktop/InJoy Firewall/"
    rm -rf /tmp/.fx-gnome
fi

rm -rf /dev/fx

rm -f /usr/lib/libfirewall.so*
rm -f /usr/lib/libfxip.so*
rm -f /usr/lib/libfxmasq.so*
rm -f /usr/lib/libfxutil.so*
rm -f /usr/lib/libipfrag.so*
rm -f /usr/lib/libfxipsec.so*
rm -f /usr/lib/libpptp.so*
rm -f /usr/lib/libpppoe.so*
rm -f /usr/lib/libdhcpd.so*
rm -f /usr/lib/libgenerate.so*
rm -f /usr/lib/libdw.so*
rm -f /usr/lib/libdwcompat.so*
rm -f /usr/share/pixmaps/gateway.png
rm -f /usr/share/pixmaps/fgui.png
rm -f /usr/share/pixmaps/logview.png
rm -f /usr/share/pixmaps/sync.png
rm -f /usr/share/pixmaps/uninstal.png

if [ "$opsystem" = "Linux" ]
then
#	clear
	echo -e $COMPLETE
fi
if [ "$opsystem" != "Linux" ]
then
	echo $COMPLETE_BSD
fi
