#!/bin/sh

#################################################
# Messages definition
#################################################

DETECT_KERN="\n You are currently running Linux kernel `uname -r`\n
	     The InJoy Firewall driver is tested only with\n
	     2.4.x and 2.6.x series of Linux kernels.\n
	     ---------------\n
	     Detecting supported kernels..."  


PRE_NO_KERN="\n > Your system does NOT have supported kernel <<\n
	     \n
	     > Kernel source matching the currently used kernel revision is\n
	     > required for the InJoy Firewall driver module to be installed.\n
	     \n
	     >>> Please contact F/X Communications for help: support@fx.dk\n"


INSTALL_DRV="Driver was installed successfully."

ABORT="> Aborting installation..."
EXIT="Exiting installation..."


NO_CRC32="\n > You don't have the crc32 module enabled.\n
	  > The module may not be necessary,\n
	  > however, if your InJoy Firewall (fx) module does not load,\n
	  > please e-mail us at support@fx.dk\n
	  \n
	  > Continuing with the installation..."

	  
NO_CARDS=">> You don't appear to have any active network cards..."


CURMOD_NOT_FOUND="\n
		  > You don't have modules folder installed for the currently running kernel.\n
		  > Installation aborted.\n"



# distro_menu() messages

DISTRO_MENU="Select your Linux distribution.\n
	    \n
	    1:  RedHat 7.2, 7.3, 8.0, 9.0\n
	    2:  Slackware 9.1\n
	    3:  Mandrake 10\n
	    4:  Fedora Core 1, 2, 3, 4, 5, 6 or Fedora 7\n
	    5:  RedHat Enterprise Linux 3.0, 4.0, 5.0\n
	    6:  Debian Sarge\n
	    ----\n
	    Q: Exit installation...\n
	    \n
	    > Other versions may also work !\n"

ASK_DISTRO="> Type the number of the distribution: "

# DISTRO SELECTION MSGS

RH="You are about to install InJoy under RedHat linux...\n\n
    Is that correct?  Press 'Y' to continue."

SLACK_91="\n > You are about to install InJoy under Slackware-9.1 linux...\n\n
	  > Is that correct? Press 'Y' to continue."

MANDR_10="\n > You are about to install InJoy under Mandrake-10.0 linux...\n\n
	  > Is that correct? Press 'Y' to continue."     

FC=" You are about to install InJoy under Fedora Core linux...\n\n
     Is that correct? Press 'Y' to continue."     

RHEL_30=" You are about to install InJoy under RedHat Enterprise Linux...\n\n
	   Is that correct? Press 'Y' to continue."

SARGE=" You are about to install InJoy under Debian Sarge linux...\n\n
	   Is that correct? Press 'Y' to continue."     

# End of distro_menu() msg	  


COPY_FAILED="failed !\n\n
	    > The driver was NOT copied.\n
	    \n
            > Please send a copy of the "$instdir"/source/logs content to\n
	    > support@fx.dk\n"


BLOCK_ALLOW="When the InJoy Firewall application is NOT running, the driver should:\n
	     ------\n
	     1: Allow all IP traffic (default)\n
	     2: Block all IP traffic"

ENABLE_DISABLE="Gateway features such as Internet Sharing require IP forwarding to be enabled.\n
		------\n
                1: Enable IP Forwarding\n
		2: Leave setting unmodified (default)"

AUTO_MANUAL="Firewall Server startup mode:\n
	     ------\n
	     1: Automatic\n
	     2: Manual"

FINISH="Installation complete, please reboot for changes to take effect."            

######## END OF DEFINITIONS #####################

if [ ! -z "$1" ]
then
	pre_distro=$1
else
	./sync -kill 2>/dev/null >/dev/null
fi

release=`uname -r`
# crelease=`uname -r | sed 's/.*\([0-9]\+\.[0-9]\+\.[0-9]\+\(-[0-9]\+\([\.\-][0-9]\+\)\?\)\?\).*/\1/'`
crelease=`uname -r | sed s@[smp$]@@g`
instdir=`pwd`
opsystem=`uname -s`
cpoptions="-a -f"


if [ "$opsystem" = "Linux" ]
then

# Checking for bakup folder
if [ -d /lib/modules/"$release"/kernel/drivers/net/fx-bak ]
then
    bak_check=`ls /lib/modules/"$release"/kernel/drivers/net/fx-bak/ |grep "\.backup"`
fi
# End check


if [ ! -d /lib/modules/"$release" ]
then
	echo -e $CURMOD_NOT_FOUND
	sleep 2
	exit 1
fi

## FUNCTION distro_menu
distro_menu () {
clear

if [ -z "$pre_distro" ]
then

	echo -e $DISTRO_MENU

	echo -n $ASK_DISTRO

	read distro

else

	distro=$pre_distro

fi

#########################################################

case $distro in
		Q)
			echo $EXIT		
			exit 1
			;;
		1)
			echo -e $RH
			read distro_answer
			if [ "$distro_answer" = Y ]
			then
				distro_choice=RH
				log_info="RedHat"
			else
			echo $ABORT			
			exit 1
			fi
			;;
		2)
			echo -e $SLACK_91		
			read distro_answer2
			if [ "$distro_answer2" = Y ]
			then
				distro_choice=Slack
				log_info="Slackware-9.1"
			else
			echo $ABORT			
			exit 1
			fi
			;;
		3)
			echo -e $MANDR_10		
			read distro_answer3
			if [ "$distro_answer3" = Y ]
			then
				distro_choice=RH
				log_info="Mandrake-10.0"
			else
			echo $ABORT			
			exit 1
			fi
			;;
		4)
			echo -e $FC
			read distro_answer3
			if [ "$distro_answer3" = Y ]
			then
				distro_choice=RH
				log_info="Fedora Core"
			else
			echo $ABORT			
			exit 1
			fi
			;;
		5)
			echo -e $RHEL_30
			read distro_answer3
			if [ "$distro_answer3" = Y ]
			then
				distro_choice=RH
				log_info="RedHat Enterprise Linux"
			else
			echo $ABORT
			exit 1
			fi
			;;
		6)
			echo -e $SARGE
			read distro_answer3
			if [ "$distro_answer3" = Y ]
			then
				distro_choice=RH
				log_info="Debian Sarge"
			else
			echo $ABORT			
			exit 1
			fi
			;;
		*)
			distro_menu
			;;


esac
}
## END OF FUNCTION distro_menu
distro_menu

clear

cd "$instdir"/source
if [ -d "$instdir"/source/logs ]
then
	rm -rf "$instdir"/source/logs
fi
mkdir "$instdir"/source/logs


echo "Before the system changes." >> ./logs/sysinfo.log
echo "------------------------" >> ./logs/sysinfo.log
echo "Distribution  : "$log_info"" >> ./logs/sysinfo.log
echo "Uname info    : `uname -a`" >> ./logs/sysinfo.log
echo "Running as     : `whoami`" >> ./logs/sysinfo.log
echo "" >> ./logs/sysinfo.log
echo "Module list   :" >> ./logs/sysinfo.log
echo "`lsmod`" >> ./logs/sysinfo.log
echo "------------------------" >> ./logs/sysinfo.log

echo -e $DETECT_KERN
sleep 1

if [ ! -z `echo $crelease | grep "2\.6"` ]
then
    mod_ext=ko
else
    mod_ext=o
fi

if [ "$crelease" = "2.4.7-10" ]
then

# Red Hat 7.2
# 2.4.7-10    i386   804083
# 2.4.7-10    i686   802068
# 2.4.7-10    athlon 812901
# 2.4.7-10smp i586   854281
# 2.4.7-10smp i686   858769
# 2.4.7-10smp athlon 855807

bootsize=`(ls -l /boot/vmlinuz-2.4.7-10 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	804083)
		kern="i386"
		modname="fx-i386.o"
	;;
	802068)
		kern="i686"
		modname="fx.o"
	;;
	812901)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.7-10smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	854281)
		smpkern="i586"
		smpmodname="fx-i586.o"
	;;
	858769)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	855807)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.4.18-3" ]
then
        
# Red Hat 7.3
# 2.4.18-3    i386   1041347
# 2.4.18-3    i686   1030147
# 2.4.18-3    athlon 1052336
# 2.4.18-3smp i586   1105332
# 2.4.18-3smp i686   1108097
# 2.4.18-3smp athlon 1103872

bootsize=`(ls -l /boot/vmlinuz-2.4.18-3 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1041347)
		kern="i386"
		modname="fx-i386.o"
	;;
	1030147)
		kern="i686"
		modname="fx.o"
	;;
	1052336)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.18-3smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1105332)
		smpkern="i586"
		smpmodname="fx-i586.o"
	;;
	1108097)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1103872)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.4.18-14" ]
then

# Red Hat 8.0
# 2.4.18-14    i586   1085191
# 2.4.18-14    i686   1125194
# 2.4.18-14    athlon 1114125
# 2.4.18-14smp i686   1206433
# 2.4.18-14smp athlon 1188204

bootsize=`(ls -l /boot/vmlinuz-2.4.18-14 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1085191)
		kern="i586"
		modname="fx-i586.o"
	;;
	1125194)
		kern="i686"
		modname="fx.o"
	;;
	1114125)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.18-14smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1206433)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1188204)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.4.20-8" ]
then

# Red Hat 9.0
# 2.4.20-8    i586   1085191
# 2.4.20-8    i686   1125194
# 2.4.20-8    athlon 1114125
# 2.4.20-8smp i686   1206433
# 2.4.20-8smp athlon ?

bootsize=`(ls -l /boot/vmlinuz-2.4.20-8 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1096436)
		kern="i586"
		modname="fx-i586.o"
	;;
	1122186)
		kern="i686"
		modname="fx.o"
	;;
	1117254)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.20-8smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1211291)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1200475)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.4.22-1.2115" ]
then

# Fedora Core I
# 2.4.22-1.2115		athlon	1239821
# 2.4.22-1.2115		i586	1228344
# 2.4.22-1.2115		i686	1252277
# 2.4.22.1.2115smp	athlon	1325447
# 2.4.22.1.2115smp	i686	1350291

bootsize=`(ls -l /boot/vmlinuz-2.4.22-1.2115.nptl | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1239821)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	1252277)
		kern="i686"
		modname="fx.o"
	;;
	1228344)
		kern="i586"
		modname="fx-i586.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.22-1.2115.nptlsmp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1350291)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1325447)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.6.5-1.358" ]
then

# Fedora Core II
# 2.6.5-1.358		i686	1199031
# 2.6.5-1.358		i586	1209805
# 2.6.5-1.358smp	i686	1282272
# 2.6.5-1.358smp	i586	1288867

bootsize=`(ls -l /boot/vmlinuz-2.6.5-1.358 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1199031)
		kern="i686"
		modname="fx.ko"
	;;
	1209805)
		kern="i586"
		modname="fx-i586.ko"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.6.5-1.358smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1282272)
		smpkern="i686"
		smpmodname="fx.ko"
	;;
	1288867)
		smpkern="i586"
		smpmodname="fx-i586.ko"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.6.9-1.667" ]
then

# Fedora Core III
# 2.6.9-1.667		i686	1405861
# 2.6.9-1.667		i586	1418253
# 2.6.9-1.667smp	i686	1370272
# 2.6.9-1.667smp	i586	1376181

bootsize=`(ls -l /boot/vmlinuz-2.6.9-1.667 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1405861)
		kern="i686"
		modname="fx.ko"
	;;
	1418253)
		kern="i586"
		modname="fx-i586.ko"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.6.9-1.667smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1370272)
		smpkern="i686"
		smpmodname="fx.ko"
	;;
	1376181)
		smpkern="i586"
		smpmodname="fx-i586.ko"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.4.21-4.EL" ]
then

# RedHat Enterprise Linux AS (Taroon)
# 2.4.21-4.EL		i686	1199031
# 2.4.21-4.EL		athlon	1209805
# 2.4.21-4.ELsmp	i686	1282272
# 2.4.21-4.ELsmp	athlon	1288867

bootsize=`(ls -l /boot/vmlinuz-2.4.21-4.EL | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1236945)
		kern="i686"
		modname="fx.o"
	;;
	1220456)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.21-4.ELsmp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1339556)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1312559)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$crelease" = "2.4.21-27.EL" ]
then

# RedHat Enterprise Linux AS (Taroon)
# 2.4.21-27.EL		i686	1248010
# 2.4.21-27.EL		athlon	1232886
# 2.4.21-27.ELsmp	i686	1357787
# 2.4.21-27.ELsmp	athlon	1322636

bootsize=`(ls -l /boot/vmlinuz-2.4.21-27.EL | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1248010)
		kern="i686"
		modname="fx.o"
	;;
	1232886)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.21-27.ELsmp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1357787)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1322636)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi


if [ "$crelease" = "2.4.26-1-386" ]
then

# Debian Sarge
# 2.4.26-1-386		i686	811877

bootsize=`(ls -l /boot/vmlinuz-2.4.26-1-386 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	811877)
		kern="i686"
		modname="fx.o"
	;;
	*)
		kern="none"
	;;
esac

success="yes"
fi


if [ "$success" = "yes" ]
then

echo "Detected $crelease kernel    : $kern"
echo "Detected ${crelease}smp kernel : $smpkern"
echo "If this is correct type Y and enter now."
read answer
if [ "$answer" != "Y" ]
then
	echo "$crelease kernel:"
	echo "        1: i386"
	echo "        2: i586"
	echo "        3: i686"
	echo "        4: Athlon"
	read answer
	case $answer in
		1)
			kern="i386"
			modname="fx-i386.$mod_ext"
		;;
		2)
			kern="i586"
			modname="fx-i586.$mod_ext"
		;;
		3)
			kern="i686"
			modname="fx.$mod_ext"
		;;
		4)
			kern="athlon"
			modname="fx-athlon.$mod_ext"
		;;
		*)
			kern="none"
			modname=
		;;
	esac
	echo "${crelease}smp kernel:"
	echo "        1: i586"
	echo "        2: i686"
	echo "        3: Athlon"
	echo
	read answer
	case $answer in
		1)
			smpkern="i586"
			smpmodname="fx-i586.$mod_ext"
		;;
		2)
			smpkern="i686"
			smpmodname="fx.$mod_ext"
		;;
		3)
			smpkern="athlon"
			smpmodname="fx-athlon.$mod_ext"
		;;
		*)
			smpkern="none"
			smpmodname=
		;;
	esac
fi

echo "Installing for $crelease kernel    : $kern"
echo "Installing for ${crelease}smp kernel : $smpkern"

cd "$instdir"/sys

if [ ! -z "$modname" ]
then
if [ -d /lib/modules/${crelease} ]
then
	if [ -d ${crelease} ]
	then
		echo "Installing /lib/modules/${crelease}/kernel/drivers/net/fx.${mod_ext}:"
		mkdir -p /lib/modules/${crelease}/kernel/drivers/net/
		cp -af ${crelease}/${modname} /lib/modules/${crelease}/kernel/drivers/net/fx.${mod_ext} 2>/dev/null
		if [ $? -eq 0 ] ; then
			driver_copied="ok"
		fi
	fi
fi
fi

if [ ! -z "$smpmodname" ]
then
if [ -d /lib/modules/${crelease}smp ]
then
	if [ -d ${crelease}smp ]
	then
		echo "Installing /lib/modules/${crelease}smp/kernel/drivers/net/fx.${mod_ext}:"
		mkdir -p /lib/modules/${crelease}smp/kernel/drivers/net/
		cp -af ${crelease}smp/${smpmodname} /lib/modules/${crelease}smp/kernel/drivers/net/fx.${mod_ext} 2>/dev/null
		if [ $? -eq 0 ] ; then
			driver_copied="ok"
		fi
	fi
fi
fi

cd ${instdir}
fi

if [ "$success" != "yes" ]
then
    echo -e $PRE_NO_KERN
	exit 1

fi

if [ "$driver_copied" = "ok" ]
then
    echo $INSTALL_DRV
else
    echo -e $COPY_FAILED
    exit 1
fi

echo -e $MAKE_CHANGE

# Create a new /etc/fx.conf with the currently loaded
# network drivers.

/sbin/lsmod | cut --delimiter=' ' --fields=1 > /tmp/modlist.tmp

if [ ! -z `uname -r | grep "2\.6"` ]
then
    /sbin/lsmod | cut --delimiter=' ' --fields=1 | sed s/_/-/g >> /tmp/modlist.tmp
fi

modlist=`sort -u /tmp/modlist.tmp`
rm -f /tmp/modlist.tmp

cd /lib/modules/${release}/kernel/drivers/net/

if [ -f /etc/fx.conf ]
then
	mv -f /etc/fx.conf /etc/fx.conf.bak
fi

for file in `find . ! -type d ! -path "*fx-bak*" -printf "%P\n"`
do

  shortname=`echo ${file} | cut --delimiter='.' --fields=1`
  for mod in ${modlist}
  do
    if [ "$mod" != "fx" ]
    then
      if [ "$mod" == `echo ${shortname} | cut --delimiter='/' --fields=2` ]
      then
        echo "${shortname}" >> /etc/fx.conf
      fi
    fi
  done

done

if [ ! -f /etc/fx.conf ]
then
	echo $NO_CARDS
	echo $ABORT
	exit 1
fi

echo -n "fx" > /etc/fx_target

cd ${instdir}

	cp -a -f "$instdir"/sys/fx_objcopy /bin/
	cp -a -f "$instdir"/sys/findsym /bin/

# Start for RH

if [ "$distro_choice" = "RH" ]
then
	cd ${instdir}

	cp -a -f "$instdir"/sys/fx /etc/init.d
	ln -sf /etc/init.d/fx /etc/rc2.d/S10fx
	ln -sf /etc/init.d/fx /etc/rc3.d/S10fx
	ln -sf /etc/init.d/fx /etc/rc4.d/S10fx
	ln -sf /etc/init.d/fx /etc/rc5.d/S10fx

	cp -a -f "$instdir"/sys/gateway /etc/init.d
	ln -sf /etc/init.d/gateway /etc/rc2.d/S90gateway
	ln -sf /etc/init.d/gateway /etc/rc3.d/S90gateway
	ln -sf /etc/init.d/gateway /etc/rc4.d/S90gateway
	ln -sf /etc/init.d/gateway /etc/rc5.d/S90gateway

# Patch the modules in fx.conf
	if [ -f /etc/fx.conf ]
	then

	  if [ -d /lib/modules/"$release"/kernel/drivers/net/fx-bak ]
	    then
		 if [ ! -z "$bak_check" ]
		 then
		    "$instdir"/Unpatch-Drivers.sh $release
		 fi
	  fi
	
	for file in `cat /etc/fx.conf`
	do
	    "$instdir"/Patch-Driver.sh $release $file
	done
	
	depmod -ae

fi
# End of the patching block

fi
# End for RH


# Start for Slack

if [ "$distro_choice" = "Slack" ]
then

	if [ -f /etc/rc.d/rc.fx ]
	then
		rm -f /etc/rc.d/rc.fx
	fi

	cp -a -f "$instdir"/sys/rc.fx /etc/rc.d/

		if [ ! -f /lib/modules/`uname -r`/kernel/lib/crc32.o ]
		then
			echo -e $NO_CRC32		

			echo "There is no crc32.o module compiled" >> "$instdir"/source/logs/sysinfo.log
		fi


# Patch the modules in fx.conf
	if [ -f /etc/fx.conf ]
	then

	  if [ -d /lib/modules/"$release"/kernel/drivers/net/fx-bak ]
	    then
		 if [ ! -z "$bak_check" ]
		 then
		    "$instdir"/Unpatch-Drivers.sh $release
		 fi
	  fi
	
	for file in `cat /etc/fx.conf`
	do
	    "$instdir"/Patch-Driver.sh $release $file
	done
	
	depmod -ae

fi
# End of the patching block

			startup_check=`sed '$!d' /etc/rc.d/rc.local`

			if [ "$startup_check" = "/etc/rc.d/rc.fx" ]
			then
				sed '$d' /etc/rc.d/rc.local >/etc/rc.d/rc.local.tmp
				mv -f /etc/rc.d/rc.local.tmp /etc/rc.d/rc.local
			fi
			echo "/etc/rc.d/rc.fx" >> /etc/rc.d/rc.local
			chmod +x /etc/rc.d/rc.local

	cp -a -f ${instdir}/sys/fx.gateway /etc/rc.d/
fi
# End of Slack


echo -e $BLOCK_ALLOW

read answer

for file in `ls -d /lib/modules/2.*`
do
    if [ ! -z `echo ${file} | grep "2\.6"` ]
    then
	tmp_mod_ext=ko
    else
	tmp_mod_ext=o
    fi

    if [ -f "$file"/kernel/drivers/net/fx."$tmp_mod_ext" ]
    then
	if [ "$answer" = "2" ]
	then
	    mv -f "$file"/kernel/drivers/net/fx."$tmp_mod_ext" "$file"/kernel/drivers/net/fx.bak
	    cat "$file"/kernel/drivers/net/fx.bak | sed s@FXENABLED1@FXENABLED0@ > "$file"/kernel/drivers/net/fx."$tmp_mod_ext"
	else
	    mv -f "$file"/kernel/drivers/net/fx."$tmp_mod_ext" "$file"/kernel/drivers/net/fx.bak
	    cat "$file"/kernel/drivers/net/fx.bak | sed s@FXENABLED0@FXENABLED1@ > "$file"/kernel/drivers/net/fx."$tmp_mod_ext"
	fi
	rm -f "$file"/kernel/drivers/net/fx.bak
    fi
done

fi
cd ${instdir}
# End of Linux type
# Handle FreeBSD specific install
if [ "$opsystem" = "FreeBSD" ]
then
	cp ${cpoptions} sys/gateway.sh /usr/local/etc/rc.d/
	if [ ! -e /etc/fx.conf ]
	then
        	# Create an /etc/fx.conf from the interface list
		for iface in `/sbin/ifconfig -l`
		do
        		found="0"
                        # Eliminate interfaces that we shouldn't firewall
			for num in 0 1 2 3 4 5 6 7 8 9
        	        do
                		if [ "$iface" = "faith${num}" ]
	                        then
        	                	found="1"
                	        fi
	                	if [ "$iface" = "lo${num}" ]
        	                then
                	        	found="1"
	                        fi
        		        	if [ "$iface" = "ppp${num}" ]
                        	then
	                        	found="1"
        	                fi
                		if [ "$iface" = "sl${num}" ]
	                        then
        	                	found="1"
                	        fi
                		if [ "$iface" = "tun${num}" ]
	                        then
        	                	found="1"
                	        fi
                		if [ "$iface" = "lp${num}" ]
	                        then
        	                	found="1"
                	        fi
                		if [ "$iface" = "ng${num}" ]
	                        then
        	                	found="1"
                	        fi
	                done
                        # If our iterface is ok add it to fx.conf
                        if [ "$found" = "0" ]
                        then
                        	echo "$iface" >> /etc/fx.conf
                        fi
		done

        fi
fi

if [ ! -e /etc/fx.conf ]
then
	touch /etc/fx.conf
fi
./Folder.sh
cp ${cpoptions} lib/* /usr/lib
/sbin/ldconfig 2> /dev/null

if [ -f /etc/fx_forward ]
then
	rm -f /etc/fx_forward
fi
clear
echo -e $ENABLE_DISABLE
read answer
if [ "$answer" = "1" ]
then
    echo 1 > /etc/fx_forward
fi

if [ -f /etc/fx_start ]
then
    rm -f /etc/fx_start
fi
clear

./Adapter.sh
clear

echo -e $AUTO_MANUAL
read answer

echo "${instdir}/gateway" > /etc/fx_start

if [ ! "$answer" = "1" ]
then
   rm -f /etc/rc?.d/S90gateway
fi

clear
echo $FINISH
