/* fx.c: FX network filter driver
 *
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <linux/errno.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>

#include <linux/if_arp.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/skbuff.h>
#include <linux/poll.h>
#include <linux/proc_fs.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 12)
#  include <linux/device.h>
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
#  include <linux/devfs_fs_kernel.h>
#endif


#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 0)
#  error This driver does not support 2.2.x and 2.3.x kernels!
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#  define KERNEL_2_4
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
#  error This driver does not support 2.5.x kernels!
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 7, 0)
#  define KERNEL_2_6
#endif

#ifdef KERNEL_2_6
#  include <linux/device.h>
#endif

#include <linux/if_bridge.h>

#ifdef KERNEL_2_6
#  include "br_private.h"
#else
#  include <br_private.h>
#endif

/* We are going to use the major number of the BDM commercial debugger
 * until we can obtain our own major number or devfs becomes the standard.
 */
#define FX_MAJOR 245

static int fx_major = -1;
static struct proc_dir_entry *fx_ent = NULL;
static int ppp_instance = -1;

#ifdef KERNEL_2_4
static devfs_handle_t devfs_handle = NULL;
int __init fx_init(struct net_device *dev);
#else
#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)
struct class_simple *fx_class = NULL;
#  else
struct class *fx_class = NULL;
#  endif
static int fx_init(struct net_device *dev);
#endif

static int fx_xmit(struct sk_buff *skb, struct net_device *dev);
static struct net_device_stats *fx_get_stats(struct net_device *dev);
void dev_mon(unsigned long instance);

#define FX_MAX_INSTANCE 20

typedef struct _fx_buf {
	struct net_device *dev;
	struct net_device_stats stats;
	struct timer_list timer;
	int timer_initialized;
	int count, ppp;
	int (*xmit) (struct sk_buff *, struct net_device *);
	int (*init) (struct net_device *);
	struct sk_buff_head inbuf;
	wait_queue_head_t inwait;
	struct sk_buff *skb_frag;
	int currentfrag;
	struct sk_buff_head outbuf;
	wait_queue_head_t outwait;
#ifdef KERNEL_2_4
	devfs_handle_t outhandle;
	devfs_handle_t inhandle;
#endif
	int fx_active;
	spinlock_t race_lock;
} FxBuf;

FxBuf *fxroot[FX_MAX_INSTANCE];
int pleaseexit[FX_MAX_INSTANCE];
char EnablementCode[] = "FXENABLED1";

int query_bridge_for = -1;

/* FXIOCINT IOCTL call */
struct fx_interface_info
{
	char 	fii_name[IFNAMSIZ];
	int	fii_index;
};

#define FXIOCINT  _IOR('f', 1, struct fx_interface_info)

/* An always useful macro */
#define MIN(a, b) (((a) < (b)) ? (a) : (b))

/* Predeclare this function for fx_init */
int fx_startup(struct net_device *, int, int, int);

int _find_instance(struct net_device *blah)
{
	int z;

	for (z = 0; z < FX_MAX_INSTANCE; z++)
		if (fxroot[z] && fxroot[z]->dev == blah)
			return z;
	return -1;
}

#pragma pack(1)
struct udphdr {
	unsigned short uh_sport;	/* source port */
	unsigned short uh_dport;	/* destination port */
	short uh_ulen;		/* udp length */
	unsigned short uh_sum;	/* udp checksum */
};

struct ip {
	unsigned char ip_hl:4;	/* header length */
	unsigned char ip_v:4;	/* version */
	unsigned char ip_tos;	/* type of service */
	unsigned short ip_len;	/* total length */
	unsigned short ip_id;	/* identification */
	unsigned short ip_off;	/* fragment offset field */
	unsigned char ip_ttl;	/* time to live */
	unsigned char ip_p;	/* protocol */
	unsigned short ip_sum;	/* checksum */
	unsigned long ip_src, ip_dst;	/* source and dest address */
};
#pragma pack()

int IsDHCP(unsigned char *buffer)
{
	/* IP Packet */
	if (buffer[12] == 0x08 && buffer[13] == 0x00) {
		struct ip *pip = (struct ip *)&buffer[14];

		if (pip->ip_p == 17) {
			struct udphdr *pudp =
			    (struct udphdr *)&buffer[14 + (pip->ip_hl * 4)];
			unsigned short dp = ntohs(pudp->uh_dport);
			unsigned short sp = ntohs(pudp->uh_sport);

			if (dp == 67 || sp == 67 || dp == 68 || sp == 68)
				return 1;
		}
	}
	return 0;
}

int is_dhcp(struct sk_buff *skb)
{
	static unsigned char buf[114];

#ifdef KERNEL_2_6
	if (skb->mac.raw) {
#else
	if (skb->mac.ethernet) {
#endif	
		int len = MIN(skb->len + 14, 100);

#ifdef KERNEL_2_6
		memcpy(buf, skb->mac.raw, 14);
#else
		memcpy(buf, skb->mac.ethernet, 14);
#endif
		memcpy(&buf[14], skb->data, len);
		return IsDHCP(buf);
	}
	return 0;
}

/* We'll make this simple and only expand two decimals */
int atoi(const char *blah)
{
	int val = 0;

	if (blah && *blah)
		val = *blah - '0';
	blah++;
	if (blah && *blah) {
		val = val * 10;
		val += *blah - '0';
	}
	return val;
}

/* Routines to deal with /proc */
static int fx_proc_read(char *buffer, char **start, off_t offset,
			int length, int *eof, void *data)
{
	struct net_bridge_port *port;
	char *name;

	*start = buffer;

	/* Signal that this is the end
	 * of the file */
	if (offset > 0)
		return 0;

	if (length > 0) {
		int bytes = MIN(length, FX_MAX_INSTANCE);

		if (ppp_instance > -1) {
			buffer[0] = (char)ppp_instance;
			ppp_instance = -1;
			
			return 1;
		} else {
			int z;

			if(query_bridge_for >= 0 && fxroot[query_bridge_for])
			{
#ifdef KERNEL_2_4
				port = fxroot[query_bridge_for]->dev->br_port;
				name = port->br->dev.name;
#else
				port = rcu_dereference(fxroot[query_bridge_for]->dev->br_port);
				name = port->br->dev->name;
#endif

				if(name[3] > '0' && name[2] < '9')
				{
					buffer[0] = name[2];
					buffer[1] = name[3];
				}
				else
				{
					buffer[0] = '0';
					buffer[1] = name[2];
				}

				buffer[2] = 'b';

				query_bridge_for = -1;

				return 3;
			}
			else/* if (length > 2) */
			{
				for (z = 0; z < bytes; z++) {

					if (fxroot[z] && fxroot[z]->fx_active)
						buffer[z] = '1';
					else
						buffer[z] = '0';
				}

				return bytes;
			}

			query_bridge_for = -1;
		}
	}

	return 0;
}

static int fx_proc_write(struct file *file, const char *user_buffer,
			 unsigned long count, void *data)
{
	char buffer[16];
	unsigned long dummy;
	
	count = MIN(count, sizeof(buffer));
	dummy = copy_from_user(buffer, user_buffer, count);

	if (count) {
		if (count == 3 && strncmp("PPP", buffer, 3) == 0) {

			struct net_device *dev =
			    kmalloc(sizeof(struct net_device), GFP_KERNEL);

			if (dev == NULL)
				return -ENOMEM;

			memset(dev, 0, sizeof(struct net_device));

			/* Initialize the device structure. */
			dev->hard_start_xmit = fx_xmit;

			/* Fill in the fields of the device structure with ethernet-generic values. */
			ether_setup(dev);

			dev->type = ARPHRD_PPP;
			dev->flags =
			    IFF_POINTOPOINT | IFF_NOARP | IFF_MULTICAST;
			    
			strcpy(dev->name, "ppp%d");

			/* Create the PPP interface */
			fx_startup(dev, 0, 1, 0);
		} else if (count == 4 && strncmp("PPP", buffer, 3) == 0) {
			int thisinstance = (int)buffer[3];

			if (thisinstance < FX_MAX_INSTANCE &&
			    fxroot[thisinstance] &&
			    fxroot[thisinstance]->dev &&
			    fxroot[thisinstance]->ppp) {
				unregister_netdev(fxroot[thisinstance]->dev);

#ifdef KERNEL_2_4
				devfs_unregister(fxroot[thisinstance]->
						 outhandle);
#else

#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)

				class_simple_device_remove(MKDEV
							   (fx_major,
							    ((thisinstance *
							      2) + 1)));

#  else

				class_device_destroy(fx_class, MKDEV(fx_major, ((thisinstance * 2) + 1)));

#  endif

#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)

				devfs_remove("fx/fxout%d", thisinstance);

#  endif				
				
#endif

				skb_queue_purge(&fxroot[thisinstance]->outbuf);
				/* fxin# is only used for ethernet interfaces */
				kfree(fxroot[thisinstance]);
				fxroot[thisinstance] = NULL;
			}
		} else if (count == 3) { /* Be careful here, since old drivers has 2 bytes buffer */
			int index = atoi(buffer);

			if (index > -1 && index < FX_MAX_INSTANCE) {
				if (buffer[2] == 'b' && fxroot[index] && fxroot[index]->dev->br_port)
					query_bridge_for = index;
				else if (buffer[2] == '0' && fxroot[index])
					fxroot[index]->fx_active = 0;
				else if (buffer[2] == '1' && fxroot[index])
					fxroot[index]->fx_active = 1;
			}
		}
	}
	return count;
}

int _find_file_instance(struct file *file)
{
	int z, len = 0;
	char *name;

	if (!file || !file->f_dentry)
		return -1;

	name = (char *)file->f_dentry->d_name.name;
	len = file->f_dentry->d_name.len;

	for (z = 0; z < len; z++) {
		if (name[z] >= '0' && name[z] <= '9') {
			int ret = atoi(&name[z]);
			if (ret > -1 && ret < FX_MAX_INSTANCE)
				return ret;
			else
				return -1;
		}
	}
	return -1;
}


/* Dealing with the in (from ethernet) device */
static unsigned int fx_in_dev_poll(struct file *file, poll_table * wait)
{
	int instance = _find_file_instance(file);

	if (instance > -1) {
		if (skb_queue_len(&(fxroot[instance]->inbuf)))
			return POLLIN | POLLRDNORM;

		poll_wait(file, &(fxroot[instance]->inwait), wait);

		if (skb_queue_len(&(fxroot[instance]->inbuf)))
			return POLLIN | POLLRDNORM;
	}
	return 0;
}

#ifdef KERNEL_2_6
#define NEED_RESCHED need_resched()
#else
#define NEED_RESCHED current->need_resched
#endif

static ssize_t fx_in_dev_write(struct file *file, const char *buf,
			       size_t count, loff_t * pos)
{
	struct sk_buff *skb;
	int instance = _find_file_instance(file);
	struct net_device *dev = fxroot[instance]->dev;
	int ret, sent_ok = 0, run = 1;
	unsigned long dummy;

	if (instance < 0)
		return -1;

	if (!fxroot[instance]->xmit && dev
	    && dev->hard_start_xmit) {
		fxroot[instance]->xmit = dev->hard_start_xmit;
		dev->hard_start_xmit = fx_xmit;
	}

	if (!fxroot[instance]->xmit)
		return -1;

	skb = alloc_skb(count, GFP_ATOMIC);
	if (skb) {
		skb_put(skb, count);

#ifdef KERNEL_2_4
		memcpy(skb->data, buf, count);
#else
		dummy = copy_from_user(skb->data, buf, count);
#endif

		skb->ip_summed = CHECKSUM_UNNECESSARY;
		skb->dev = fxroot[instance]->dev;

		/*
		 * Next code is modified inject() function from
		 * net/core/pktgen.c
		 */
		while(run && !sent_ok) {

		    if(!netif_queue_stopped(dev)) {

			atomic_inc(&skb->users);

#ifdef KERNEL_2_6
		    retry:
			ret = fxroot[instance]->xmit(skb, dev);

			if(likely(ret == NETDEV_TX_OK)) {
			    sent_ok = 1;

			} else if(ret == NETDEV_TX_LOCKED && 
				(dev->features & NETIF_F_LLTX)) {
			    cpu_relax();
			    goto retry;
#else
			ret = fxroot[instance]->xmit(skb, dev);
			if(!ret) {
			    sent_ok = 1;
#endif
			} else {
			    atomic_dec(&skb->users);
			    sent_ok = 0;
			}
		    } /* if queue is not stopped */
		    else {
			sent_ok = 0;
		    }

		    if(signal_pending(current))
			break;

		    if(netif_queue_stopped(dev) || NEED_RESCHED) {

			do {
			    if(	signal_pending(current) ||
				!netif_running(dev)) {
				run = 0;
				break;
			    }
			    if(NEED_RESCHED)
				schedule();
			    else
				do_softirq();
			} while(netif_queue_stopped(dev));
		    }
		} /* while we running */

		kfree_skb(skb);

		if(!sent_ok)
		    printk(KERN_CRIT "packet wasn't send\n");

	} /* if skb allocated */

	return 0;
}

static ssize_t fx_in_dev_read(struct file *file, char *buf,
			      size_t count, loff_t * pos)
{
	int instance = _find_file_instance(file);
	struct sk_buff *skb;
	unsigned long dummy;

	if (instance < 0)
		return -1;

	if ((skb = skb_dequeue(&(fxroot[instance]->inbuf))) != NULL) {
		int len = MIN(count, skb->len);

#ifdef KERNEL_2_6
		if (skb->mac.raw) {
#else
		if (skb->mac.ethernet) {
#endif
			len = MIN(count, skb->len + 14);

			if (len > 14)
#ifdef KERNEL_2_4
				memcpy(buf, skb->mac.ethernet, 14);
#else
				dummy = copy_to_user(buf, skb->mac.raw, 14);
#endif
			else
				len = -1;
		}
		/* Copy the data into the destination buffer */
		if (len > 14)
#ifdef KERNEL_2_4
			memcpy(&buf[14], skb->data, len);
#else
			dummy = copy_to_user(&buf[14], skb->data, len);
#endif
		else
			len = -1;

		kfree_skb(skb);

		return len;
	}
	return 0;
}

static loff_t fx_in_dev_lseek(struct file *file, loff_t offset, int origin)
{
	return -ESPIPE;
}

static int fx_in_dev_open(struct inode *inode, struct file *file)
{
#ifdef KERNEL_2_4
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

static int fx_in_dev_release(struct inode *inode, struct file *file)
{
#ifdef KERNEL_2_4
	MOD_DEC_USE_COUNT;
#endif
	return 0;
}

static int fx_dev_ioctl(int instance, struct inode *inode, struct file *file,
			unsigned int cmd, unsigned long arg)
{
	struct fx_interface_info info;
	
	if (fxroot[instance] != NULL && fxroot[instance]->dev != NULL)
	{
		memset(&info, 0, sizeof(struct fx_interface_info));
		
		memcpy(info.fii_name, fxroot[instance]->dev->name, IFNAMSIZ);
		info.fii_index = instance;
		
		if (copy_to_user((char *)arg, (char *)&info, sizeof(struct fx_interface_info)))
			return -EFAULT;
	}
	else
		return -ENOENT;
			
	return 0;
}

static int fx_in_dev_ioctl(struct inode *inode, struct file *file,
			   unsigned int cmd, unsigned long arg)
{
	int instance = _find_file_instance(file);

	if (instance < 0)
		return -1;
		
	if (cmd == FXIOCINT)
		return fx_dev_ioctl(instance, inode, file, cmd, arg);

	return 0;
}

static struct file_operations fx_in_dev_fops = {
      llseek:fx_in_dev_lseek,
      read:fx_in_dev_read,
      write:fx_in_dev_write,
      poll:fx_in_dev_poll,
      ioctl:fx_in_dev_ioctl,
      open:fx_in_dev_open,
      release:fx_in_dev_release,
};

/* Dealing with the out (from kernel TCP/IP) device */
static unsigned int fx_out_dev_poll(struct file *file, poll_table * wait)
{
	int instance = _find_file_instance(file);

	if (instance > -1) {
		if (skb_queue_len(&(fxroot[instance]->outbuf)))
			return POLLIN | POLLRDNORM;

		poll_wait(file, &(fxroot[instance]->outwait), wait);

		if (skb_queue_len(&(fxroot[instance]->outbuf)))
			return POLLIN | POLLRDNORM;
	}
	return 0;
}

static ssize_t fx_out_dev_write(struct file *file, const char *buf,
				size_t count, loff_t * pos)
{
	struct sk_buff *skb;
	int instance = _find_file_instance(file);
	unsigned long dummy;

	if (instance < 0 || count < 15)
		return -1;

	skb = alloc_skb(count, GFP_ATOMIC);
	if (skb) {
		int result;
		short proto;

#ifdef KERNEL_2_4
		skb->mac.ethernet = (struct ethhdr *)skb_put(skb, count);
		memcpy(skb->mac.ethernet, buf, count);
		proto = *((short *)&buf[12]);
		skb->data = (char *)skb->mac.ethernet + ETH_HLEN;
#else
		skb->mac.raw = skb_put(skb, count);
		dummy = copy_from_user(skb->mac.raw, buf, count);
		dummy = copy_from_user((char *)&proto, &buf[12], sizeof(proto));
		skb->data = skb->mac.raw + ETH_HLEN;
#endif
		skb->nh.raw = skb->data;
		skb->protocol = proto;
		skb->dev = fxroot[instance]->dev;

		result = netif_rx(skb);
		if (fxroot[instance]->ppp) {
			fxroot[instance]->stats.rx_packets++;
			fxroot[instance]->stats.rx_bytes += count;
		}

		switch (result) {
		case NET_RX_DROP:
			printk(KERN_INFO
			       "Packet dropped to interface fx%d while offering up.\n",
			       instance);
			break;
		case NET_RX_CN_LOW:
			printk(KERN_INFO
			       "Low congestion on interface fx%d while offering up.\n",
			       instance);
			break;
		case NET_RX_CN_MOD:
			printk(KERN_INFO
			       "Moderate congestion on interface fx%d while offering up.\n",
			       instance);
			break;
		case NET_RX_CN_HIGH:
			printk(KERN_INFO
			       "High congestion on interface fx%d while offering up.\n",
			       instance);
			break;
		}
		return count;
	}
	return 0;
}

static ssize_t fx_out_dev_read(struct file *file, char *buf,
			       size_t count, loff_t * pos)
{
	int instance = _find_file_instance(file);
	struct sk_buff *skb;
	char sbuf[1600], *buf_ptr;
	unsigned long dummy;

	if (instance < 0)
		return -1;

	/* We don't have any fragments so we grab a new packet */
	if ((skb = skb_dequeue(&(fxroot[instance]->outbuf))) != NULL) {
		int len = MIN(count, skb->len);

		buf_ptr = skb->data;

		/* Claculate checksum if necessary */
		
		if(skb->ip_summed == CHECKSUM_HW && len < sizeof(sbuf))
		{
		    skb_copy_and_csum_dev(skb, sbuf);
		    buf_ptr = sbuf;
		}

		/* Copy the data into the destination buffer */
#ifdef KERNEL_2_4
		memcpy(buf, buf_ptr, len);
#else
		dummy = copy_to_user(buf, buf_ptr, len);
#endif
		kfree_skb(skb);

		return len;
	}
	return 0;
}

static loff_t fx_out_dev_lseek(struct file *file, loff_t offset, int origin)
{
	return -ESPIPE;
}

static int fx_out_dev_open(struct inode *inode, struct file *file)
{
#ifdef KERNEL_2_4
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

static int fx_out_dev_release(struct inode *inode, struct file *file)
{
#ifdef KERNEL_2_4
	MOD_DEC_USE_COUNT;
#endif
	return 0;
}

static int fx_out_dev_ioctl(struct inode *inode, struct file *file,
			    unsigned int cmd, unsigned long arg)
{
	int instance = _find_file_instance(file);

	if (instance < 0)
		return -1;
		
	if (cmd == FXIOCINT)
		return fx_dev_ioctl(instance, inode, file, cmd, arg);

	return 0;
}

static struct file_operations fx_out_dev_fops = {
      llseek:fx_out_dev_lseek,
      read:fx_out_dev_read,
      write:fx_out_dev_write,
      poll:fx_out_dev_poll,
      ioctl:fx_out_dev_ioctl,
      open:fx_out_dev_open,
      release:fx_out_dev_release,
};

/* Generic */
static int fx_generic_open(struct inode *inode, struct file *file)
{
	int len = 0;
	char *name;

	if (!file || !file->f_dentry)
		return -ENODEV;

	name = (char *)file->f_dentry->d_name.name;
	len = file->f_dentry->d_name.len;

	/* On a non-devfs system we have to set the file_operation
	 * structs ourselves since devfs_register() doesn't.
	 */
	if (len > 3 && strncmp("fxin", name, 4) == 0) {
		file->f_op = &fx_in_dev_fops;
		return fx_in_dev_open(inode, file);
	} else if (len > 4 && strncmp("fxout", name, 5) == 0) {
		file->f_op = &fx_out_dev_fops;
		return fx_out_dev_open(inode, file);
	}
	return -ENODEV;
}

static struct file_operations fx_generic_fops = {
      open:fx_generic_open,
};

static int local_unregister_dev(struct net_device *x, int nonlock)
{
	int instance = _find_instance(x);

	if (instance > -1) {
		if (fxroot[instance]->timer_initialized)
			del_timer(&fxroot[instance]->timer);

#ifdef KERNEL_2_4
		devfs_unregister(fxroot[instance]->inhandle);
		devfs_unregister(fxroot[instance]->outhandle);
#else


#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)

		class_simple_device_remove(MKDEV(fx_major, (instance * 2)));
		class_simple_device_remove(MKDEV
					   (fx_major, ((instance * 2) + 1)));

#  else

		class_device_destroy(fx_class, MKDEV(fx_major, (instance * 2)));
		class_device_destroy(fx_class, MKDEV(fx_major, ((instance * 2) + 1)));

#  endif

#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
		
		devfs_remove("fx/fxin%d", instance);
		devfs_remove("fx/fxout%d", instance);
		
#  endif
		
#endif

		kfree(fxroot[instance]);
		fxroot[instance] = NULL;
	}
#ifdef KERNEL_2_4
	MOD_DEC_USE_COUNT;
#endif

	if(nonlock)
	{
		return unregister_netdevice(x);
	}

	unregister_netdev(x);
	return 0;
}

/* The next five functions are for overloading subdrivers */
void fx_unregister_netdev(struct net_device *x)
{
	local_unregister_dev(x, 0);
}

int fx_unregister_netdevice(struct net_device *x)
{
	return local_unregister_dev(x, 1);
}

int fx_receive_prep(struct sk_buff *skb)
{
	int instance = _find_instance(skb->dev);
	
	/* May need to free the skb here */
	if (instance < 0) {
		kfree_skb(skb);
		return 1;
	}

	/* There have been cases where the timer fails
	 * so we double check here.
	 */
	if (!fxroot[instance]->xmit)
		dev_mon(instance);

	if (EnablementCode[9] == '0' && !fxroot[instance]->fx_active
	    && !is_dhcp(skb)) {
		kfree_skb(skb);
		return 1;
	}

	if (fxroot[instance]->fx_active || fxroot[instance]->ppp) {
		skb_queue_tail(&(fxroot[instance]->inbuf), skb);
		wake_up_interruptible(&(fxroot[instance]->inwait));
		return 2;
	} else
		return 0;
}

#ifdef KERNEL_2_6
int fx_netif_receive_skb(struct sk_buff *skb)
{
	int rc;
	
	rc = fx_receive_prep(skb);
	
	if(rc == 0)
	{
		return netif_receive_skb(skb);
	}
	else if(rc == 1)
	{
		return NET_RX_DROP;
	}
	else
	{
		return NET_RX_SUCCESS;
	}
}
#endif

void fx_netif_rx(struct sk_buff *skb)
{
	int rc;

	rc = fx_receive_prep(skb);

	if(!rc)
		netif_rx(skb);
}

/* For oldstyle drivers */
struct net_device *fx_init_etherdev(struct net_device *x, int y)
{
	struct net_device *dev = kmalloc(sizeof(struct net_device), GFP_KERNEL);
	memset(dev, 0, sizeof(struct net_device));

	if (y) {
		dev->priv = kmalloc(y, GFP_KERNEL);
		memset(dev->priv, 0, y);
	}
#ifdef KERNEL_2_4
	MOD_INC_USE_COUNT;
#endif

	ether_setup(dev);
	fx_startup(dev, 1, 0, 0);
	return dev;
}

/* For new style drivers */
int fx_register_netdev(struct net_device *dev)
{
	return fx_startup(dev, 0, 0, 0);
}

/* non-block version (used by e1000) */
int fx_register_netdevice(struct net_device *dev)
{
	return fx_startup(dev, 0, 0, 1);
}

/* Dealing with the interface */
#ifdef KERNEL_2_4
int __init fx_init(struct net_device *dev)
#else
static int fx_init(struct net_device *dev)
#endif
{
	int instance = _find_instance(dev);

	if (instance > -1) {
		int res = 0;

		if (fxroot[instance]->init)
			res = fxroot[instance]->init(dev);

		fxroot[instance]->xmit = dev->hard_start_xmit;
		dev->hard_start_xmit = fx_xmit;

		return res;
	}

	return 0;
}

static int fx_xmit(struct sk_buff *skb, struct net_device *dev)
{
	int instance = _find_instance(dev);

	/* May need to free the skb here */
	if (instance < 0) {
		kfree_skb(skb);
		return -1;
	}

	if (fxroot[instance]->fx_active || fxroot[instance]->ppp) {
		if (fxroot[instance]->ppp) {
			fxroot[instance]->stats.tx_packets++;
			fxroot[instance]->stats.tx_bytes += skb->len;
		}
		skb_queue_tail(&(fxroot[instance]->outbuf), skb);
		wake_up_interruptible(&(fxroot[instance]->outwait));
	} else if (fxroot[instance]->xmit)
		return fxroot[instance]->xmit(skb, fxroot[instance]->dev);
	else
		kfree_skb(skb);
	return 0;
}

static struct net_device_stats *fx_get_stats(struct net_device *dev)
{
	int instance = _find_instance(dev);

	if (instance < 0)
		return NULL;
	return &fxroot[instance]->stats;
}

/* Monitor the net_device structure waiting for function initialization */
void dev_mon(unsigned long instance)
{
	struct net_device *dev = fxroot[instance]->dev;

	if (dev->hard_start_xmit) {
		fxroot[instance]->xmit = dev->hard_start_xmit;
		dev->hard_start_xmit = fx_xmit;

		del_timer(&fxroot[instance]->timer);
		fxroot[instance]->count = 0;
		return;
	}

	fxroot[instance]->count++;

	/* We only try to check for 60 seconds */
	if (fxroot[instance]->count == 60) {
		del_timer(&fxroot[instance]->timer);
		fxroot[instance]->count = 0;
		return;
	}

	mod_timer(&fxroot[instance]->timer, HZ);
}

int fx_startup(struct net_device *dev, int old, int ppp, int nonlock)
{
#ifdef KERNEL_2_4
	char name[20];
#endif
	int count = 0, err = -1;
	
	/* Find first free instance */
	while (fxroot[count]) {
		count++;
		if (count == FX_MAX_INSTANCE)
			return -1;
	}

	if (ppp)
		ppp_instance = count;

	/* Allocate a buffer for the current instance */
	fxroot[count] = kmalloc(sizeof(FxBuf), GFP_KERNEL);
	memset(fxroot[count], 0, sizeof(FxBuf));

	fxroot[count]->ppp = ppp;
	/*
	 *  dev field initialized at the end somehow.
	 *  It's impossible now, because of fx_startup runningduring this call.
	 *  But otherwise in case of failure initialization _findinstance could
	 *  return index of broken device struct.
	*/
	fxroot[count]->dev = dev;
	fxroot[count]->init = dev->init;

	init_waitqueue_head(&(fxroot[count]->outwait));
	skb_queue_head_init(&(fxroot[count]->outbuf));

	if (!ppp) {
		init_waitqueue_head(&(fxroot[count]->inwait));
		skb_queue_head_init(&(fxroot[count]->inbuf));

#ifdef KERNEL_2_4
		sprintf(name, "fxin%d", count);

		fxroot[count]->inhandle =
		    devfs_register(devfs_handle, name, DEVFS_FL_DEFAULT,
				   fx_major, (count * 2),
				   S_IFCHR | S_IRUSR | S_IWUSR, &fx_in_dev_fops,
				   NULL);
#else

#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)

		class_simple_device_add(fx_class, MKDEV(fx_major, (count * 2)),
					NULL, "fx/fxin%d", count);
					
#  else

#    if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 15)

		class_device_create(fx_class, NULL, MKDEV(fx_major, (count * 2)), 
				    NULL, "fx/fxin%d", count);

#    else

		class_device_create(fx_class, MKDEV(fx_major, (count * 2)), 
				    NULL, "fx/fxin%d", count);

#    endif

#  endif					

#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)

		devfs_mk_cdev(MKDEV(fx_major, (count * 2)),
			      S_IFCHR | S_IRUSR | S_IWUSR, "fx/fxin%d",
			      count);
			      
#  endif			      
			      
#endif
	}
#ifdef KERNEL_2_4
	sprintf(name, "fxout%d", count);

	fxroot[count]->outhandle =
	    devfs_register(devfs_handle, name, DEVFS_FL_DEFAULT, fx_major,
			   (count * 2) + 1, S_IFCHR | S_IRUSR | S_IWUSR,
			   &fx_out_dev_fops, NULL);
#else

#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)

	class_simple_device_add(fx_class, MKDEV(fx_major, ((count * 2) + 1)),
				NULL, "fx/fxout%d", count);

#  else

#    if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 15)

	class_device_create(fx_class, NULL, MKDEV(fx_major, (count * 2) + 1), 
			    NULL, "fx/fxout%d", count);

#    else

	class_device_create(fx_class, MKDEV(fx_major, (count * 2) + 1), 
			    NULL, "fx/fxout%d", count);
			    
#    endif

#  endif

#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)

	devfs_mk_cdev(MKDEV(fx_major, ((count * 2) + 1)),
		      S_IFCHR | S_IRUSR | S_IWUSR, "fx/fxout%d", count);
		      
#  endif		      
		      
#endif

	/* Find a name for this unit */
	err = dev_alloc_name(dev, dev->name);

	if (err < 0)
		return err;
		
	printk(KERN_INFO "fx: registered device \"%s\", device index for InJoy is %d.\n",
		dev->name, count + 1);

	if (!old && !ppp)
		dev->init = fx_init;

	if (ppp)
		dev->get_stats = fx_get_stats;

	/* Ok we create an ethernet device */
	if(nonlock)
	{
		err = register_netdevice(dev);
		if (err != 0)
			return err;
	}
	else
	{
		if (register_netdev(dev) != 0)
			return -EIO;
	}

	fxroot[count]->timer_initialized = old;

	if (old) {
		init_timer(&fxroot[count]->timer);
		fxroot[count]->timer.function = dev_mon;
		fxroot[count]->timer.data = (unsigned long)count;
		fxroot[count]->timer.expires = HZ;
		fxroot[count]->count = 0;
		add_timer(&fxroot[count]->timer);
	}

	return 0;
}

int fx_init_module(void)
{
#ifdef KERNEL_2_6
	int err;
#endif

	printk(KERN_INFO "F/X Communications Network Filter Driver, Copyright (c) 2000-2004\n");

	/* Clear the instance list */
	memset(fxroot, 0, sizeof(FxBuf *) * FX_MAX_INSTANCE);
	memset(pleaseexit, 0, sizeof(int) * FX_MAX_INSTANCE);

	/* We also create entries in /dev */

#ifdef KERNEL_2_4

	if ((fx_major =
	     devfs_register_chrdev(FX_MAJOR, "fx", &fx_generic_fops)) < 0) {
		printk(KERN_ERR "fx: unable to get major %d (%d)\n", FX_MAJOR,
		       fx_major);
		return -EIO;
	} else if (fx_major == 0) {
		/* In the case that devfs isn't installed we will
		 * get a return code of zero so set fx_major to the
		 * requested major.
		 */
		fx_major = FX_MAJOR;
	}

	/* We'll put them under the /dev/fx directory */
	devfs_handle = devfs_mk_dir(NULL, "fx", NULL);

#else

	fx_major = FX_MAJOR;

	if ((err = register_chrdev(FX_MAJOR, "fx", &fx_generic_fops)) != 0) {
		printk(KERN_ERR "fx: unable to get major %d (%d)\n", FX_MAJOR,
		       err);
		return -EIO;
	}

#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)

	fx_class = class_simple_create(THIS_MODULE, "fx");
	
#  else

	fx_class = class_create(THIS_MODULE, "fx");

#  endif	

	if (IS_ERR(fx_class)) {
		printk(KERN_ERR "fx: unable to register simple class %d (%d)\n",
		       FX_MAJOR, (int)(PTR_ERR(fx_class)));
		return -EIO;
	}

#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)

	devfs_mk_dir("fx");
	
#  endif	

#endif

	fx_ent = create_proc_entry("fx", S_IFREG | S_IRUGO | S_IWUGO,
				   &proc_root);

	if (fx_ent) {
		fx_ent->read_proc = fx_proc_read;
		fx_ent->write_proc = fx_proc_write;
	}

	printk(KERN_INFO "Initialization complete. FX Major is %u.\n",
	       fx_major);

	return 0;
}

void fx_cleanup_module(void)
{
	int z;

	for (z = 0; z < FX_MAX_INSTANCE; z++) {
		if (fxroot[z])
			kfree(fxroot[z]);
	}

#ifdef KERNEL_2_4
	devfs_unregister(devfs_handle);
	devfs_unregister_chrdev(fx_major, "fx");
#else

#  if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 12)

	class_simple_destroy(fx_class);
	
#  else

	class_destroy(fx_class);

#  endif	
	
	unregister_chrdev(fx_major, "fx");

#endif

	remove_proc_entry("fx", &proc_root);

	printk(KERN_INFO "FX Network Filter Driver successfully unloaded.\n");
}

/* Credits: ALL the helpful beta-testers that worked with us in the community.
 *          All developers that helped getting this component solid.
 *          Brian Smith for his original open source contribution.
 */

MODULE_AUTHOR("F/X Communications <info@fx.dk>");
MODULE_DESCRIPTION("F/X Communications Network Filter");
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif


/* 8390.c gets statically linked into the kernel in Linux 2.4.x, and
 * any NE2000 based cards that use it will bypass our frame redirection,
 * so we need to export these functions ourself but to avoid conflicts
 * we must rename the functions, which is what the #defines below are for.
 */
#ifdef MODULE
#undef MODULE
#endif
#undef ei_open
#define ei_open fx_ei_open
#undef ei_close
#define ei_close fx_ei_close
#undef ei_interrupt
#define ei_interrupt fx_ei_interrupt
#undef ei_tx_timeout
#define ei_tx_timeout fx_ei_tx_timeout
#undef ethdev_init
#define ethdev_init fx_ethdev_init
#undef NS8390_init
#define NS8390_init fx_NS8390_init
#undef netif_rx
#define netif_rx fx_netif_rx
#undef MODULE_LICENSE
#define MODULE_LICENSE(a)
#include "8390.c"

EXPORT_SYMBOL(fx_netif_rx);
EXPORT_SYMBOL(fx_register_netdev);
EXPORT_SYMBOL(fx_register_netdevice);
EXPORT_SYMBOL(fx_unregister_netdev);
EXPORT_SYMBOL(fx_unregister_netdevice);
EXPORT_SYMBOL(fx_init_etherdev);
#ifdef KERNEL_2_6
EXPORT_SYMBOL(fx_netif_receive_skb);
#endif

module_init(fx_init_module);
module_exit(fx_cleanup_module);
